package org.expeditee.auth.tags;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

import org.expeditee.items.Text;

public enum AuthenticationTag {
	Username ("txtUsername", System.getProperty("user.name")), 
	Password ("txtPassword", System.getProperty("user.password")), 
	PasswordAgain ("txtPasswordAgain", System.getProperty("user.password.again")), 
	NewPassword ("txtNewPassword", System.getProperty("user.new.password")), 
	NewPasswordAgain ("txtNewPasswordAgain", System.getProperty("user.new.password.again")), 
	Email ("txtEmail", System.getProperty("user.email")), 
	EmailAgain ("txtEmailAgain", System.getProperty("user.email.again")),
	TrustedUserOne ("txtTrustedUserOne", System.getProperty("user.colleague.one")), 
	TrustedUserTwo ("txtTrustedUserTwo", System.getProperty("user.colleague.two")), 
	Frameset ("txtFrameset", System.getProperty("user.toshare.frameset")), 
	IntergalacticNumber ("txtIntergalacticNumber", System.getProperty("user.intergalacticnumber")), 
	PasswordSliceOne ("txtPasswordPieceOne", System.getProperty("user.password.piece.one")), 
	PasswordSliceTwo ("txtPasswordPieceTwo", System.getProperty("user.password.piece.two")); 
	
	public String val;
	public String def;

	private AuthenticationTag(String tag, String def) {
		this.val = tag;
		this.def = def;		
	}
	
	public static Optional<Map<AuthenticationTag, String>> fetchUserData(Collection<Text> from, boolean useDefaults, AuthenticationTag... tags) {
		final Map<AuthenticationTag, String> fetchedData = new HashMap<AuthenticationTag, String>();
		
		for (AuthenticationTag tag: tags) {
			for (Text canditate: from) {
				// use the canditate if it exists
				if (canditate.hasData(tag.val)) {
					if (tag.val.equals("txtUsername")) {
						fetchedData.put(tag, canditate.getText().toLowerCase());
					} else {
						fetchedData.put(tag, canditate.getText());
					}
					break;
				}
				// otherwise try use the default
				String property = tag.def;
				if (property != null && !property.isEmpty()) { 
					fetchedData.put(tag, useDefaults ? property : null); 
				}
			}
		}
		
		if (fetchedData.entrySet().size() == tags.length) {
			return Optional.of(fetchedData);
		} else { 
			return Optional.empty();
		}
	}
}
